<?php

namespace Drupal\mnc_notify\Plugin\Notifier;

use Drupal\Component\Render\FormattableMarkup;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\message\MessageInterface;
use Drupal\message_notify\Exception\MessageNotifyException;
use Drupal\message_notify\Plugin\Notifier\MessageNotifierBase;
use Drupal\user\UserInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a message notifier for Message Notification Center.
 *
 * @Notifier(
 *   id = "mnc_email",
 *   title = @Translation("MNC Email"),
 *   description = @Translation("Send message via email."),
 *   viewModes = {
 *     "message_notify_email_subject",
 *     "message_notify_email_body"
 *   }
 * )
 */
class MessageNotificationCenterEmail extends MessageNotifierBase {

  use StringTranslationTrait;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The mail manager.
   *
   * @var \Drupal\Core\Mail\MailManagerInterface
   */
  protected $maiLManager;

  /**
   * Constructs the plugin.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Logger\LoggerChannelInterface $logger
   *   The message_notify logger channel.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The rendering service.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   * @param \Drupal\Core\Mail\MailManagerInterface $mail_manager
   *   The mail manager.
   * @param \Drupal\message\MessageInterface|null $message
   *   (optional) The message entity. This is required when sending or
   *   delivering a notification. If not passed to the constructor, use
   *   ::setMessage().
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, LoggerChannelInterface $logger, EntityTypeManagerInterface $entity_type_manager, RendererInterface $renderer, ModuleHandlerInterface $module_handler, LanguageManagerInterface $language_manager, MailManagerInterface $mail_manager, MessageInterface $message = NULL) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $logger, $entity_type_manager, $renderer, $message);

    $this->moduleHandler = $module_handler;
    $this->languageManager = $language_manager;
    $this->maiLManager = $mail_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition, MessageInterface $message = NULL) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('logger.channel.mnc_notify'),
      $container->get('entity_type.manager'),
      $container->get('renderer'),
      $container->get('module_handler'),
      $container->get('language'),
      $container->get('plugin.manager.mail'),
      $message
    );
  }

  /**
   * {@inheritdoc}
   */
  public function send(): bool {
    $output = [];
    $message = $this->message;
    $options = $this->configuration;

    $recipients = [];
    if ($options['mnc_recipients']) {
      $recipients = [];
      foreach ($options['mnc_recipients'] as &$recipient) {
        // Skip if we cannot determine a recipient email address.
        if (!isset($recipient['mail']) && !$recipient['user'] instanceof AccountInterface) {
          continue;
        }

        // If no email address was specified, use the email address configured
        // in the user's account.
        if (!isset($recipient['mail'])) {
          $recipient['mail'] = $recipient['user']->getEmail();
        }
        $recipients[] = $recipient;
      }
    }
    elseif ($message->hasField('field_mnc_recipients')) {
      if ($message->get('field_mnc_recipients')->isEmpty()) {
        // No recipients, so return without further processing.
        return TRUE;
      }
      else {
        foreach ($message->get('field_mnc_recipients') as $recipient) {
          $user = $recipient->entity;
          if ($user instanceof UserInterface) {
            $recipients[] = [
              'user' => $user,
              'mail' => $user->getEmail(),
            ];
          }
        }
      }
    }
    else {
      // No recipients field found or provided, so return without further
      // processing.
      return TRUE;
    }

    $results = TRUE;
    foreach ($recipients as $recipient) {
      $message->recipient = $recipient;
      foreach ($this->pluginDefinition['viewModes'] as $view_mode => $value) {
        $view_builder = $this->entityTypeManager->getViewBuilder('message');
        foreach ($this->pluginDefinition['viewModes'] as $view_mode) {
          $build = $view_builder->view($this->message, $view_mode);
          $output[$view_mode] = $this->renderer->renderPlain($build);
        }

        // Allow other modules to alter the email text.
        $context = [
          'message' => $message,
          'view_mode' => $view_mode,
        ];
        \Drupal::moduleHandler()->alter('mnc_notify_email_text', $output[$view_mode], $context);
      }
      $result = $this->deliver($output);
      $results = $results && $result;
      $this->postSend($result, $output);
    }

    return $results;
  }

  /**
   * {@inheritdoc}
   */
  public function deliver(array $output = []) {
    $message = $this->message;
    $options = $this->configuration;

    $account = $message->recipient['user'];
    $mail = $message->recipient['mail'];
    $from = $options['from'] ?: NULL;

    // Determine the language to use for the email.
    if (!$this->configuration['language override']) {
      $language = $account->getPreferredLangcode();
    }
    else {
      $language = $this->message->language()->getId();
    }

    // The subject in an email cannot contain HTML, so strip it.
    $output['message_notify_email_subject'] = strip_tags($output['message_notify_email_subject']);

    // Pass the message entity along to hook_drupal_mail().
    $output['message_entity'] = $message;

    $result =  $this->mailManager->mail(
      'mnc_notify',
      $message->getTemplate()->id(),
      $mail,
      $language,
      $output,
      $from
    );

    return $result['result'];
  }

  /**
   * {@inheritdoc}
   */
  public function postSend($result, array $output = []) {
    $message = $this->message;
    $options = $this->configuration;

    $save = FALSE;
    if (!$result) {
      $this->logger->error($this->t('Could not send message using @title to user ID @uid.'), [
        '@label' => $this->pluginDefinition['title'],
        '@uid' => $message->recipient->uid,
      ]);
      if ($options['save on fail']) {
        $save = TRUE;
      }
    }
    elseif ($result && $options['save on success']) {
      $save = TRUE;
    }

    if ($options['rendered fields']) {
      // Save the rendered output into matching fields.
      foreach ($this->pluginDefinition['viewModes'] as $view_mode => $mode) {
        if (empty($options['rendered fields'][$view_mode])) {
          throw new MessageNotifyException(new FormattableMarkup(
            'The rendered view mode @mode cannot be saved to field, as there is not a matching one.',
            ['@mode' => $mode['label']]
          ));
        }
        $field_name = $options['rendered fields'][$view_mode];

        if (!$message->hasField($field_name)) {
          throw new MessageNotifyException(new FormattableMarkup(
            'Field @field does not exist.',
            ['@field' => $field_name]
          ));
        }

        if (empty($message->get($field_name)->format)) {
          $message->set($field_name, $output[$view_mode]);
        }
        else {
          // Get the format from the template. We assume the first delta is the
          // same as the rest.
          $format = $message->getTemplate()->getText()[0]['format'];
          $message->set($field_name, ['value' => $output[$view_mode], 'format' => $format]);
        }
      }
    }

    if ($save) {
      // Todo: how should we handle failed message saves?
      // $message->save();
    }
  }

}
