<?php

namespace Drupal\mnc\Controller;

use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Ajax\SettingsCommand;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Render\RendererInterface;
use Drupal\mnc\Services\MessagesHelper;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Provides controller methods for Message Notification Center.
 */
class MncController extends ControllerBase {

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The messages helper service.
   *
   * @var \Drupal\mnc\Services\MessagesHelper
   */
  protected $messagesHelper;

  /**
   * Constructs a \Drupal\mnc\Controller\MncController object.
   *
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The rendered service.
   * @param \Drupal\mnc\Services\MessagesHelper $messages_helper
   *   The messages helper service.
   */
  public function __construct(RendererInterface $renderer, MessagesHelper $messages_helper) {
    $this->renderer = $renderer;
    $this->messagesHelper = $messages_helper;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('renderer'),
      $container->get('mnc.messages_helper')
    );
  }

  /**
   * Returns information for displaying a message notification block.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request.
   *
   * @return array|\Symfony\Component\HttpFoundation\Response
   *   A renderable array for displaying the message notification block or an
   *   ajax response with commands in case of an XML HTTP request.
   */
  public function block(Request $request) {
    $config = $this->config('mnc.settings');
    $view_display = $config->get('view_display') ?? 'mnc:messages';
    [$view_id, $view_display_name] = explode(':', $view_display);
    $build = array(
      '#theme' => 'mnc_ajax',
      '#view' => views_embed_view($view_id, $view_display_name, $this->currentUser()->id()),
    );

    $settings['mnc'] = ['unread_count' => $this->messagesHelper->getUnreadMessagesCount()];

    if ($request->isXmlHttpRequest()) {
      $response = new AjaxResponse();
      // Order of commands is important, if we don't change settings _before_
      // replacing content the attached behaviors will still use the old
      // settings.
      $response->addCommand(new SettingsCommand($settings));
      $response->addCommand(new ReplaceCommand('.message-view', $build));

      return $response;
    }
    else {
      $build['#attached']['drupalSettings'] = $settings;
      return $build;
    }
  }

  /**
   * Marks given message(s) as read.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   JSON response with new unread messages count.
   */
  public function markRead(Request $request): JsonResponse {
    foreach ($this->getMessagesFromRequest($request) as $message) {
      $this->messagesHelper->markRead($message);
    }

    return new JsonResponse([
      'unread' => $this->messagesHelper->getUnreadMessagesCount(),
    ]);
  }

  /**
   * Marks given message(s) as unread.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   JSON response with new unread messages count.
   */
  public function markUnread(Request $request): JsonResponse {
    foreach ($this->getMessagesFromRequest($request) as $message) {
      $this->messagesHelper->markUnread($message);
    }

    return new JsonResponse([
      'unread' => $this->messagesHelper->getUnreadMessagesCount(),
    ]);
  }

  /**
   * Marks all message(s) for the current user as read.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   JSON response with new unread messages count.
   */
  public function markAllRead(): JsonResponse {
    $this->messagesHelper->markAllRead();

    return new JsonResponse([
      'unread' => $this->messagesHelper->getUnreadMessagesCount(),
    ]);
  }

  /**
   * Marks all message(s) for the current user as unread.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   JSON response with new unread messages count.
   */
  public function markAllUnread(): JsonResponse {
    $this->messagesHelper->markAllUnread();

    return new JsonResponse([
      'unread' => $this->messagesHelper->getUnreadMessagesCount(),
    ]);
  }

  /**
   * Returns the number of unread messages for the current user.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   JSON response with new unread messages count.
   */
  public function unreadCount(): JsonResponse {
    return new JsonResponse([
      'unread' => $this->messagesHelper->getUnreadMessagesCount(),
    ]);
  }

  /**
   * Returns the messages extracted from the request data.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request to extract the messages from.
   *
   * @return \Drupal\message\MessageInterface[]
   *   The messages extracted from the request.
   */
  protected function getMessagesFromRequest(Request $request): array {
    $message_ids = [];
    if ($request->get('message_id')) {
      if (is_array($request->get('message_id'))) {
        $message_ids = $request->get('message_id');
      }
      else {
        $message_ids = [$request->get('message_id')];
      }
    }

    try {
      /** @var \Drupal\message\MessageInterface[] $messages */
      return $this->entityTypeManager->getStorage('message')
        ->loadMultiple($message_ids);
    }
    catch (InvalidPluginDefinitionException | PluginNotFoundException $e) {
      watchdog_exception('mnc', $e);
    }

    return [];
  }

}
