<?php

namespace Drupal\mnc\Form\Config;

use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\views\Views;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a configuration form for Message Notification Center.
 */
class MncSettingsForm extends ConfigFormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a \Drupal\mnc\Form\Config\MncSettingsForm object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(ConfigFactoryInterface $config_factory, EntityTypeManagerInterface $entity_type_manager) {
    parent::__construct($config_factory);

    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['mnc.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'mnc_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $config = $this->config('mnc.settings');

    $form['link_inner_prefix'] = [
      '#type' => 'textfield',
      '#title' => $this->t('The inner prefix of the messages unread link'),
      '#description' => $this->t('Content is run through $this->t(). This can be useful for removing the text and using only an icon.'),
      '#default_value' => $config->get('link_inner_prefix') ?? 'unread messages',
    ];
    $form['include_anonymous_messages'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Include anonymous messages (uid = 0) for all users?'),
      '#description' => $this->t("this can be useful if you want to send messages to all users. Note: the read status will not be changed during viewing, in order to change it and also dismiss it's notification count you will have to edit that message."),
      '#default_value' => $config->get('include_anonymous_messages') ?? FALSE,
    ];
    $form['qtip_classes'] = [
      '#type' => 'textfield',
      '#title' => $this->t('The classes used for the qtip'),
      '#default_value' => $config->get('qtip_classes') ?? 'qtip-light qtip-shadow',
    ];
    $form['qtip_width'] = [
      '#type' => 'number',
      '#title' => $this->t('The width used for the qtip'),
      '#description' => $this->t('Please note that you will have to adjust the .qtip {max-width} in css too!'),
      '#default_value' => $config->get('qtip_width') ?? 350,
    ];
    $form['view_display'] = [
      '#type' => 'select',
      '#title' => $this->t('The display used'),
      '#description' => $this->t('Change it back to master if you switch views'),
      '#options' => Views::getViewsAsOptions(FALSE, 'enabled', NULL, TRUE, TRUE),
      '#default_value' => $config->get('view_display') ?? 'mnc:messages',
    ];
    $form['view_display_unread'] = [
      '#type' => 'select',
      '#title' => $this->t('The display used for unread messages'),
      '#description' => $this->t('Change it back to master if you switch views'),
      '#options' => Views::getViewsAsOptions(FALSE, 'enabled', NULL, TRUE, TRUE),
      '#default_value' => $config->get('view_display_unread') ?? 'mnc:messages_unread',
    ];
    $form['view_display_notifications_unread'] = [
      '#type' => 'select',
      '#title' => $this->t('The display used for unread notification messages'),
      '#description' => $this->t('Change it back to master if you switch views'),
      '#options' => Views::getViewsAsOptions(FALSE, 'enabled', NULL, TRUE, TRUE),
      '#default_value' => $config->get('view_display_notifications_unread') ?? 'mnc:notification_unread',
    ];
    $form['count_callback'] = [
      '#type' => 'textfield',
      '#title' => $this->t('The callback function used for populating the notification icon'),
      '#description' => $this->t('The callback function gets a user object passed. To limit the count to only unread notifications, instead of unread messages use: count_unread_notifications.'),
      '#default_value' => $config->get('count_callback') ?? 'mnc.messages_helper:getUnreadMessageCount',
    ];
    $form['flag'] = [
      '#type' => 'select',
      '#title' => $this->t('The flag machine name to be used for marking read messages'),
      '#options' => $this->getAvailableMessageFlags(),
      '#default_value' => $config->get('flag') ?? 'mnc_read_message',
    ];
    $form['cache_lifetime_count_unread'] = [
      '#type' => 'number',
      '#title' => $this->t('Cache lifetime for the unread count (seconds)'),
      '#description' => $this->t('When using poll mode be aware that this value should be lower than the actual polling time. Use 0 for no cache.'),
      '#default_value' => $config->get('cache_lifetime_count_unread') ?? 30,
    ];
    $form['expire_seconds'] = [
      '#type' => 'number',
      '#title' => $this->t('Client side cache lifetime for the unread count (seconds)'),
      '#description' => $this->t('Use 0 for no cache.'),
      '#default_value' => $config->get('expire_seconds') ?? 60,
    ];
    $form['refresh_interval'] = [
      '#type' => 'number',
      '#title' => $this->t('Client side refresh interval for the unread count (seconds)'),
      '#description' => $this->t("If a user has modernizr enabled and uses multiple tabs, the refreshing takes care of updating the count if another tab changed the localstorage. Use 0 for no refreshing."),
      '#default_value' => $config->get('refresh_interval') ?? 5,
    ];
    $form['refresh_interval_poll'] = [
      '#type' => 'number',
      '#title' => $this->t('Client side refresh interval for the unread count (seconds)'),
      '#description' => $this->t('Used if modernzir and/ or localstorage is not available. Use 0 for no additional polling.'),
      '#default_value' => $config->get('refresh_interval_poll') ?? 60,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->configFactory()->getEditable('mnc.settings');

    $config->set('link_inner_prefix', $form_state->getValue('link_inner_prefix'))
      ->set('include_anonymous_messages', $form_state->getValue('include_anonymous_messages'))
      ->set('qtip_classes', $form_state->getValue('qtip_classes'))
      ->set('qtip_width', $form_state->getValue('qtip_width'))
      ->set('view_display', $form_state->getValue('view_display'))
      ->set('view_display_unread', $form_state->getValue('view_display_unread'))
      ->set('view_display_notifications_unread', $form_state->getValue('view_display_notifications_unread'))
      ->set('count_callback', $form_state->getValue('count_callback'))
      ->set('flag', $form_state->getValue('flag'))
      ->set('cache_lifetime_count_unread', $form_state->getValue('cache_lifetime_count_unread'))
      ->set('expire_seconds', $form_state->getValue('expire_seconds'))
      ->set('refresh_interval', $form_state->getValue('refresh_interval'))
      ->set('refresh_interval_poll', $form_state->getValue('refresh_interval_poll'))
      ->save();

    parent::submitForm($form, $form_state);
  }

  /**
   * Returns all enabled flags that apply to message entities.
   *
   * @return array
   *   Associative array of flag labels keyed by flag ID.
   */
  protected function getAvailableMessageFlags(): array {
    $flags = [];

    try {
      $flag_entities = $this->entityTypeManager->getStorage('flag')
        ->loadByProperties(['entity_type' => 'message', 'status' => TRUE]);

      foreach ($flag_entities as $flag_entity) {
        $flags[$flag_entity->id()] = $flag_entity->label();
      }
    }
    catch (InvalidPluginDefinitionException | PluginNotFoundException $e) {
      watchdog_exception('mnc', $e);
    }

    return $flags;
  }

}
