<?php

namespace Drupal\mnc\Plugin\Block;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a block for displaying the Message Notification Center.
 *
 * @Block(
 *   id = "mnc",
 *   admin_label = @Translation("Message Notification Center")
 * )
 */
class MncBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * Constructs a \Drupal\mnc\Plugin\Block\MncBlock object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ConfigFactoryInterface $config_factory, AccountInterface $current_user) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->configFactory = $config_factory;
    $this->currentUser = $current_user;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory'),
      $container->get('current_user')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function build(): array {
    $config = $this->configFactory->get('mnc.settings');
    $view_display = $config->get('view_display') ?? 'mnc:messages';
    [$view_id, $view_display_name] = explode(':', $view_display);
    $build = array(
      '#theme' => 'mnc_ajax',
      '#view' => views_embed_view($view_id, $view_display_name, $this->currentUser->id()),
    );
    $this->addJs($build);
    $build['#attached']['library'][] = 'core/drupal.ajax';

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function access(AccountInterface $account, $return_as_object = FALSE) {
    $access = AccessResult::allowedIfHasPermission($account, 'access message notification center');
    return $return_as_object ? $access : $access->isAllowed();
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheMaxAge(): int {
    return 0;
  }

  /**
   * Adds required javascript libraries to the block.
   *
   * Also provides hooks for you to alter data:
   *  - hook_mnc_js_alter()
   *  - hook_mnc_css_alter()
   *  - hook_mnc_libraries_alter()
   *
   * @param $build
   *   The renderable array for the block.
   */
  protected function addJs(&$build): void {
    if (\Drupal::currentUser()->isAuthenticated()) {
      // Attach our libraries.
      $libraries = ['mnc/core', 'mnc/qtip'];

      // Attach our module settings.
      $config = $this->configFactory->get('mnc.settings');
      $view_display = $config->get('view_display');
      [$view_id] = explode(':', $view_display);
      $settings = [
        'view' => $view_id ?? 'mnc',
        'view_css_selector' => '.view-' . str_replace('_', '-', $view_id ?? 'mnc'),
        'qtip_classes' => $config->get('mnc_qtip_classes') ?? 'qtip-light qtip-shadow',
        'qtip_width' => $config->get('mnc_qtip_width') ?? 350,
        'expire_seconds' => $config->get('mnc_expire_seconds') ?? 60,
        'refresh_interval' => $config->get('mnc_refresh_interval') ?? 5,
        'refresh_interval_poll' => $config->get('mnc_refresh_interval_poll') ?? 60,
      ];

      // Allow for alterations by other modules.
      \Drupal::moduleHandler()->alter('mnc_libraries', $libraries);
      \Drupal::moduleHandler()->alter('mnc_js_settings', $settings);

      // Actually attach the libraries and settings to the build array.
      foreach ($libraries as $library) {
        $build['#attached']['library'][] = $library;
      }
      $build['#attached']['drupalSettings']['mnc'] = $settings;
    }
  }

}
