<?php

namespace Drupal\mnc\Services;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Logger\LoggerChannelTrait;
use Drupal\Core\Session\AccountInterface;
use Drupal\flag\FlagInterface;
use Drupal\flag\FlagServiceInterface;
use Drupal\message\MessageInterface;
use Drupal\views\ViewExecutable;
use Drupal\views\Views;

/**
 * Provides helper methods around messages for a user.
 */
class MessagesHelper {

  use LoggerChannelTrait;

  const UNREAD_MESSAGES_DISPLAY_ID = 'view_display_unread';

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The 'mnc' cache bin.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The flag service.
   *
   * @var \Drupal\flag\FlagServiceInterface
   */
  protected $flag;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The callback resolver.
   *
   * @var \Drupal\mnc\Services\CallbackResolver
   */
  protected $callbackResolver;

  /**
   * Constructs a \Drupal\mnc\Services\MessagesHelper object.
   *
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache
   *   The 'mnc' cache bin.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory.
   * @param \Drupal\flag\FlagServiceInterface $flag
   *   The flag service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_channel_factory
   *   The logger channel factory.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\mnc\Services\CallbackResolver $callback_resolver
   *   The callback resolver.
   */
  public function __construct(AccountInterface $current_user, CacheBackendInterface $cache, ConfigFactoryInterface $config_factory, FlagServiceInterface $flag, LoggerChannelFactoryInterface $logger_channel_factory, ModuleHandlerInterface $module_handler, CallbackResolver $callback_resolver) {
    $this->currentUser = $current_user;
    $this->cache = $cache;
    $this->configFactory = $config_factory;
    $this->flag = $flag;
    $this->setLoggerFactory($logger_channel_factory);
    $this->moduleHandler = $module_handler;
    $this->callbackResolver = $callback_resolver;
  }

  /**
   * Returns the number of unread messages for a user.
   *
   * @param \Drupal\Core\Session\AccountInterface|null $user
   *   The user for whom to count the unread messages. Defaults to the current
   *   user.
   * @param bool $refresh
   *   If true the previous result is fetched from cache. If false the result is
   *   calculated. Defaults to false.
   *
   * @return int
   *   The number of unread messages for the user.
   */
  public function getUnreadMessagesCount(?AccountInterface $user = NULL, bool $refresh = FALSE): int {
    if (NULL === $user) {
      $user = $this->currentUser;
    }

    // Allow projects to use a custom count callback by setting one in config.
    $callback = $this->configFactory->get('mnc.settings')->get('mnc_count_callback');
    $callable = $this->callbackResolver->resolve($callback);
    if ($callable && is_callable($callable)) {
      return call_user_func($callable, $user, $refresh);
    }

    $config = $this->configFactory->get('mnc.settings');

    $cache_key = 'mnc_count_unread:' . $user->id();
    $cache = $this->cache->get($cache_key);

    $cache_lifetime_count_unread = $config->get('cache_lifetime_count_unread') ?? 30;
    if (!$refresh && $cache_lifetime_count_unread > 0 && $cache && (time() < $cache->expire) ) {
      return $cache->data;
    }
    else {
      /*
       * Build the view with needed configuration but don't execute it usually
       * instead take the count_query object and execute that. This is done to
       * achieve a performance benefit.
       */
      $view_display = $config->get(static::UNREAD_MESSAGES_DISPLAY_ID) ?? 'mnc:messages_unread';
      [$view_id, $view_display_name] = explode(':', $view_display);
      $view = Views::getView($view_id);
      $view->setDisplay($view_display_name);

      if ($config->get('include_anonymous_messages')) {
        // Include anonymous user recipients messages.
        $view->setArguments(['0+' . $user->id()]);
      }
      else {
        $view->setArguments([$user->id()]);
      }
      $view->build($view_display_name);
      $count = $this->getMessageCountFromView($view);

      $this->cache->set($cache_key, $count, time() + $cache_lifetime_count_unread);
      return $count;
    }
  }

  /**
   * Returns the number of messages retrieved by executing a views executable.
   *
   * @param \Drupal\views\ViewExecutable $view
   *   The view to get the number of retrieved messages from.
   *
   * @return int
   *   Total number of messages retrieved.
   */
  public function getMessageCountFromView(ViewExecutable $view): int {
    // We want to get the total number of rows from the pager.
    $view->get_total_rows = TRUE;
    // Load only 1 item for performance.
    $view->setItemsPerPage(1);
    $view->execute();
    return $view->getPager()->getTotalItems();
  }

  /**
   * Returns whether a given message is read by a given user.
   *
   * @param \Drupal\message\MessageInterface $message
   *   The message to check.
   * @param \Drupal\Core\Session\AccountInterface|null $user
   *   The user to check for. Defaults to current user.
   *
   * @return bool
   *   Whether the message was read by the user.
   */
  public function isRead(MessageInterface $message, AccountInterface $user = NULL): bool {
    if (NULL === $user) {
      $user = $this->currentUser;
    }

    $flag = $this->getReadFlag();
    if ($flag instanceof FlagInterface) {
      return $flag->isFlagged($message, $user);
    }

    return FALSE;
  }

  /**
   * Marks a message as read by a given user.
   *
   * @param \Drupal\message\MessageInterface $message
   *   The message to mark as read.
   * @param \Drupal\Core\Session\AccountInterface|null $user
   *   The user the mark the message read for. Defaults to current user.
   */
  public function markRead(MessageInterface $message, AccountInterface $user = NULL): void {
    if (NULL === $user) {
      $user = $this->currentUser;
    }

    $flag = $this->getReadFlag();
    if ($flag instanceof FlagInterface && !$this->flag->getFlagging($flag, $message, $user)) {
      $this->flag->flag($flag, $message, $user);
      if ($this->moduleHandler->moduleExists('rules')) {
        // TODO: Rules integration.
//        rules_invoke_event('mnc_mark_read', $user, $message->id());
      }
    }
  }

  /**
   * Marks a message as unread by a given user.
   *
   * @param \Drupal\message\MessageInterface $message
   *   The message to mark as unread.
   * @param \Drupal\Core\Session\AccountInterface|null $user
   *   The user the mark the message unread for. Defaults to current user.
   */
  public function markUnread(MessageInterface $message, AccountInterface $user = NULL): void {
    if (NULL === $user) {
      $user = $this->currentUser;
    }

    $flag = $this->getReadFlag();
    if ($flag instanceof FlagInterface && $this->flag->getFlagging($flag, $message, $user)) {
      $this->flag->unflag($flag, $message, $user);
      if ($this->moduleHandler->moduleExists('rules')) {
        // TODO: Rules integration.
//        rules_invoke_event('mnc_mark_unread', $user, $message->id());
      }
    }
  }

  /**
   * Marks all messages as read by a given user.
   *
   * TODO: This method is highly inefficient. Takes seconds to mark hundreds of
   *   messages. Probably need to provide some sort of feedback (throbber, etc.)
   *   to the user. Could potentially be improved by going to the DB directly
   *   rather than executing a view and then flagging each item individually,
   *   but we would lose flexibility in the process.
   *
   * @param \Drupal\Core\Session\AccountInterface|null $user
   *   The user the mark the messages read for. Defaults to current user.
   * @param bool $first_page_only
   *   Whether to limit the marking to only the first page of view results.
   *
   * @return ViewExecutable|null
   *   The Views executable used for determining the messages for the user or
   *   null if no 'read' flag was found.
   */
  public function markAllRead(AccountInterface $user = NULL, bool $first_page_only = FALSE): ?ViewExecutable {
    if (NULL === $user) {
      $user = $this->currentUser;
    }

    $flag = $this->getReadFlag();
    if ($flag instanceof FlagInterface) {
      $view_display = $this->configFactory->get('mnc.settings')->get(static::UNREAD_MESSAGES_DISPLAY_ID) ?? 'mnc:messages_unread';
      [$view_id, $view_display_name] = explode(':', $view_display);
      $view = Views::getView($view_id);

      foreach ($this->getMessagesFromView($view, $view_display_name, $user, $first_page_only) as $message) {
        $this->flag->flag($flag, $message->_entity, $user);
        if ($this->moduleHandler->moduleExists('rules')) {
          // TODO: Rules integration.
//        rules_invoke_event('mnc_mark_read', $user, $message->id());
        }
      }

      return $view;
    }

    return NULL;
  }

  /**
   * Marks all messages as unread by a given user.
   *
   * TODO: This method is highly inefficient. Takes seconds to mark hundreds of
   *   messages. Probably need to provide some sort of feedback (throbber, etc.)
   *   to the user. Could potentially be improved by going to the DB directly
   *   rather than executing a view and then flagging each item individually,
   *   but we would lose flexibility in the process.
   *
   * @param \Drupal\Core\Session\AccountInterface|null $user
   *   The user the mark the messages unread for. Defaults to current user.
   * @param bool $first_page_only
   *   Whether to limit the marking to only the first page of view results.
   *
   * @return ViewExecutable|null
   *   The Views executable used for determining the messages for the user or
   *   null if no 'read' flag was found.
   */
  public function markAllUnread(AccountInterface $user = NULL, bool $first_page_only = FALSE): ?ViewExecutable {
    if (NULL === $user) {
      $user = $this->currentUser;
    }

    $flag = $this->getReadFlag();
    if ($flag instanceof FlagInterface) {
      $view_display = $this->configFactory->get('mnc.settings')->get('mnc_view_display') ?? 'mnc:messages';
      [$view_id, $view_display_name] = explode(':', $view_display);
      $view = Views::getView($view_id);

      foreach ($this->getMessagesFromView($view, $view_display_name, $user, $first_page_only) as $message) {
        $this->flag->unflag($flag, $message->_entity, $user);
        if ($this->moduleHandler->moduleExists('rules')) {
          // TODO: Rules integration.
//        rules_invoke_event('mnc_mark_read', $user, $message->id());
        }
      }

      return $view;
    }

    return NULL;
  }

  /**
   * Gets the 'read' flag from config.
   *
   * @return \Drupal\flag\FlagInterface|null
   *   The 'read' flag entity or null if no valid flag is configured.
   */
  protected function getReadFlag(): ?FlagInterface {
    $flag_id = $this->configFactory->get('mnc.settings')->get('flag') ?? 'mnc_read_message';
    $flag = $this->flag->getFlagById($flag_id);

    if ($flag instanceof FlagInterface) {
      return $flag;
    }
    else {
      $this->getLogger('mnc')->warning(
        'The defined read flag {flag} does not exist.',
        ['flag' => $flag_id]
      );
    }

    return NULL;
  }

  /**
   * Gets the messages for a user by executing a Views executable.
   *
   * @param \Drupal\views\ViewExecutable $view
   *   The views executable to execute.
   * @param string $view_display_name
   *   The machine name of the display to use from the view.
   * @param \Drupal\Core\Session\AccountInterface|null $user
   *   The user for which to execute the view. Default to the current user.
   * @param bool $first_page_only
   *   Whether to limit the marking to only the first page of view results.
   *
   * @return \Drupal\message\MessageInterface[]
   *   Array of messages as returned by the view.
   */
  protected function getMessagesFromView(ViewExecutable $view, string $view_display_name, AccountInterface $user = NULL, bool $first_page_only = FALSE): array {
    if (NULL === $user) {
      $user = $this->currentUser;
    }
    $messages = [];

    if (!$first_page_only) {
      // Mark all notifications for this user, not just the first page of results.
      $view->setItemsPerPage(0);
    }
    $view->setDisplay($view_display_name);
    $view->setArguments([$user->id()]);
    $view->preExecute();
    $view->execute();

    foreach ($view->result as $message) {
      $messages[] = $message->_entity;
    }

    return $messages;
  }

}
